package sun.font;

import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.Rectangle;
import java.awt.geom.Point2D;

public final class CompositeStrike extends FontStrike
{
    static final int SLOTMASK = 16777215;
    private CompositeFont compFont;
    private PhysicalStrike[] strikes;
    int numGlyphs;
    
    CompositeStrike(CompositeFont compFont, FontStrikeDesc desc) {
        this.numGlyphs = 0;
        this.compFont = compFont;
        this.desc = desc;
        this.disposer = new FontStrikeDisposer(this.compFont, desc);
        if (desc.style != this.compFont.style) {
            this.algoStyle = true;
            if ((desc.style & 0x1) == 0x1 && (this.compFont.style & 0x1) == 0x0) {
                this.boldness = 1.33f;
            }
            if ((desc.style & 0x2) == 0x2 && (this.compFont.style & 0x2) == 0x0) {
                this.italic = 0.7f;
            }
        }
        this.strikes = new PhysicalStrike[this.compFont.numSlots];
    }
    
    PhysicalStrike getStrikeForGlyph(int n) {
        return this.getStrikeForSlot(n >>> 24);
    }
    
    PhysicalStrike getStrikeForSlot(int n) {
        if (n >= this.strikes.length) {
            n = 0;
        }
        PhysicalStrike physicalStrike = this.strikes[n];
        if (physicalStrike == null) {
            physicalStrike = (PhysicalStrike)(PhysicalStrike)this.compFont.getSlotFont(n).getStrike(this.desc);
            this.strikes[n] = physicalStrike;
            return physicalStrike;
        }
        return physicalStrike;
    }
    
    @Override
    public int getNumGlyphs() {
        return this.compFont.getNumGlyphs();
    }
    
    @Override
    StrikeMetrics getFontMetrics() {
        if (this.strikeMetrics == null) {
            StrikeMetrics strikeMetrics = new StrikeMetrics();
            for (int i = 0; i < this.compFont.numMetricsSlots; ++i) {
                strikeMetrics.merge(this.getStrikeForSlot(i).getFontMetrics());
            }
            this.strikeMetrics = strikeMetrics;
        }
        return this.strikeMetrics;
    }
    
    @Override
    void getGlyphImagePtrs(int[] array, long[] array2, int n) {
        int i = this.getStrikeForSlot(0).getSlot0GlyphImagePtrs(array, array2, n);
        if (i == n) {
            return;
        }
        for (; i < n; ++i) {
            array2[i] = this.getStrikeForGlyph(array[i]).getGlyphImagePtr(array[i] & 0xFFFFFF);
        }
        return;
    }
    
    @Override
    long getGlyphImagePtr(int n) {
        return this.getStrikeForGlyph(n).getGlyphImagePtr(n & 0xFFFFFF);
    }
    
    @Override
    void getGlyphImageBounds(int n, Point2D.Float float1, Rectangle rectangle) {
        this.getStrikeForGlyph(n).getGlyphImageBounds(n & 0xFFFFFF, float1, rectangle);
    }
    
    @Override
    Point2D.Float getGlyphMetrics(int n) {
        return this.getStrikeForGlyph(n).getGlyphMetrics(n & 0xFFFFFF);
    }
    
    @Override
    Point2D.Float getCharMetrics(char c) {
        return this.getGlyphMetrics(this.compFont.getMapper().charToGlyph(c));
    }
    
    @Override
    float getGlyphAdvance(int n) {
        return this.getStrikeForGlyph(n).getGlyphAdvance(n & 0xFFFFFF);
    }
    
    @Override
    float getCodePointAdvance(int n) {
        return this.getGlyphAdvance(this.compFont.getMapper().charToGlyph(n));
    }
    
    @Override
    Rectangle2D.Float getGlyphOutlineBounds(int n) {
        return this.getStrikeForGlyph(n).getGlyphOutlineBounds(n & 0xFFFFFF);
    }
    
    @Override
    GeneralPath getGlyphOutline(int n, float n2, float n3) {
        GeneralPath glyphOutline = this.getStrikeForGlyph(n).getGlyphOutline(n & 0xFFFFFF, n2, n3);
        if (glyphOutline == null) {
            return new GeneralPath();
        }
        return glyphOutline;
    }
    
    @Override
    GeneralPath getGlyphVectorOutline(int[] array, float n, float n2) {
        GeneralPath generalPath = null;
        int i = 0;
        while (i < array.length) {
            int n3;
            for (, n3 = array[i] >>> 24; i < array.length && array[i + 1] >>> 24 == n3; ++i) {}
            int n4 = i - i + 1;
            int[] array2 = new int[n4];
            for (int j = 0; j < n4; ++j) {
                array2[j] = (array[j] & 0xFFFFFF);
            }
            GeneralPath glyphVectorOutline = this.getStrikeForSlot(n3).getGlyphVectorOutline(array2, n, n2);
            if (generalPath == null) {
                generalPath = glyphVectorOutline;
            }
            else if (glyphVectorOutline != null) {
                generalPath.append(glyphVectorOutline, false);
            }
            else {
                continue;
            }
        }
        if (generalPath == null) {
            return new GeneralPath();
        }
        return generalPath;
    }
}
